package com.ibm.pureQuery.demo.myclasses;

import java.io.StringWriter;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ibm.pdq.runtime.exception.DataRuntimeException;
import com.ibm.pdq.runtime.handlers.ResultHandler;

public class HTMLHandler implements ResultHandler<String>
{

  private DocumentBuilderFactory documentBuilderFactory_;
  private DocumentBuilder domBuilder_;
  private Transformer transformer_;

  public HTMLHandler ()
  {
    documentBuilderFactory_ = DocumentBuilderFactory.newInstance ();
    try {
      domBuilder_ = documentBuilderFactory_.newDocumentBuilder ();
    }
    catch (ParserConfigurationException e) {
      throw new RuntimeException ("Unable to initilize document builder " + e.getMessage (), e);
    }
    TransformerFactory transformerFactory = TransformerFactory.newInstance ();
    try {
      transformer_ = transformerFactory.newTransformer ();
      transformer_.setOutputProperty (OutputKeys.INDENT, "yes");
      transformer_.setOutputProperty (OutputKeys.OMIT_XML_DECLARATION, "yes");
    }
    catch (TransformerConfigurationException e) {
      throw new RuntimeException (e.getMessage (), e);
    }
  }

  public String handle (ResultSet resultSet)
  {
    Document document = domBuilder_.newDocument ();

    // Create root element
    Element rootElement = document.createElement ("html");
    rootElement.setAttribute ("xmlns", "http://www.w3.org/TR/REC-html40");
    document.appendChild (rootElement);

    Element headElement = document.createElement ("head");
    rootElement.appendChild (headElement);
    Element titleElement = document.createElement ("title");
    titleElement.setTextContent ("HTML Table Printer");
    rootElement.appendChild (titleElement);

    Element bodyElement = document.createElement ("body");
    rootElement.appendChild (bodyElement);

    generatedTable (resultSet, bodyElement, document);

    return transformXML (document);
  }

  private void generatedTable (ResultSet resultSet, Element bodyElement, Document document)
  {
    try {
      ResultSetMetaData resultSetMetaData = resultSet.getMetaData ();
      int columnCount = resultSetMetaData.getColumnCount ();
      Element tableElement = document.createElement ("TABLE");
      tableElement.setAttribute ("border", "1");
      bodyElement.appendChild (tableElement);
      Element headerRowElement = document.createElement ("TR");
      tableElement.appendChild (headerRowElement);

      for (int index = 0; index < columnCount; index++) {
        Element headerElement = document.createElement ("TH");
        headerElement.setTextContent (resultSetMetaData.getColumnLabel (index + 1));
        headerRowElement.appendChild (headerElement);
      }
      while (resultSet.next ()) {
        Element rowElement = document.createElement ("TR");
        tableElement.appendChild (rowElement);
        for (int index = 0; index < columnCount; index++) {
          Element columnElement = document.createElement ("TD");
          columnElement.setTextContent (resultSet.getString (index + 1));
          tableElement.appendChild (columnElement);
        }
      }

    }
    catch (SQLException sqle) {
      throw new RuntimeException ("Unable to access the result " + sqle.getMessage (), sqle);
    }
  }

  private String transformXML (Document document)
  {
    StringWriter stringWriter = new StringWriter ();
    try {
      transformer_.transform (new DOMSource (document), new StreamResult (stringWriter));
    }
    catch (TransformerException e) {
      //throw new DataRuntimeException ("Unable to generate Metadata annotation " + e.getMessage (), e);
    }

    return stringWriter.toString ();
  }
}
